<?php

namespace App\Controller\Project;

/**
 * Class Backlinks
 * @package App\Controller\Project
 * @author Artsiom Malkovich (skype: artmalk)
 */
class Backlinks extends \App\Controller\Admin
{
    public function importAction()
    {
        $project = $this->getProject(1);
        $addedCount = 0;
        $errors = array();
        $uploaded = false;
        if (!$project->id) {
            $this->redirect(PROJECT_FOLDER);
        }
        $lastParser = $project->getLastParser();

        if (($lastParser && $lastParser->getStatus() == \App\Model\Parser::STATUS_FINISH) || !$lastParser) {
            try {
                if ($this->request()->isPost()) {
                    if (!isset($_FILES['file'])) {
                        throw new \Exception(\t('UPLOAD_SELECT_FILE_EXCEPTION'));
                    }

                    $avoidMulti = $this->request()->post('avoid');

                    $file = $_FILES['file'];

                    if ($file['error']) {
                        throw new \Exception(\t('UPLOAD_ERROR_UPLOAD'));
                    }

                    $ext = end(explode('.', $file['name']));
                    if ($ext != 'csv') {
                        throw new \Exception(\t('UPLOAD_INVALID_EXTENSION'));
                    }
                    $uploaded = true;
                    $contents = file_get_contents($file['tmp_name']);
                    $contents = str_replace(array("\r\n", "\r"), array("\n", "\n"), $contents);
                    $lines = explode("\n", $contents);
                    foreach ($lines as $num => $line) {
                        if ($num == (count($lines) - 1) && !trim($line)) {
                            continue; //skip last line if empty
                        }

                        $csvLine = str_getcsv($line);
                        if (isset($csvLine[0])) {
                            $link = trim($csvLine[0]);
                        }
                        else {
                            $link = null;
                        }

                        if ($link=== 'Link') {
                            continue;
                        }

                        //TODO: merge import and add
                        if (\App\Model\Backlink::isValid($link)) {
                            $link = \App\Model\Backlink::formatLink($link);
                            if ($project->isBacklinkExists($link)) {
                                $errors[$num] = 'Link on line ' . $num . ' already exists';
                                continue;
                            }
                            if ($avoidMulti && $project->isDomainExists($link)) {
                                $errors[$num] = 'Link with domain '.\App\Model\Backlink::getHostWithoutWWW($link).' on line ' . $num . ' already exists';
                                continue;
                            }
                            $backlink = \R::dispense('backlinks');
                            $backlink->link = $link;
                            $backlink->projects_id = 1;
                            \R::store($backlink);
                            $project->newBacklink($link);
                            $addedCount++;

                        } else {
                            if (trim($line) == '') {
                                $errors[$num] = 'Empty link on line ' . $num;
                            } else {
                                $errors[$num] = 'Invalid link on line ' . $num;
                            }
                        }
                    }
                }
            } catch (\Slim\Exception $e) {
                $this->app->flashNow('upload-error', $e->getMessage());
            }
        } else {
            $this->app->flashNow('upload-error', \t('CHANGE_LINKS_PARSER_NOT_FINISHED'));
        }
        $this->render('project/backlinks/import', array(
            'layout' => 'layouts/project.php',
            'project' => $project,
            'errors' => $errors,
            'addedCount' => $addedCount,
            'uploaded' => $uploaded
        ));


    }

    public function addAction()
    {
        /** @var \App\Model\Project $project */
        $project = $this->getProject(1);
        if (!$project->id) {
            $this->redirect(PROJECT_FOLDER);
        }
        $lastParser = $project->getLastParser();
        $alerts = array();
        if (($lastParser && $lastParser->getStatus() == \App\Model\Parser::STATUS_FINISH) || !$lastParser ) {
            $links = explode("\n", trim($this->request()->post('links')));
            $avoidMulti = $this->request()->post('avoid');
            foreach ($links as $link) {
                if (\App\Model\Backlink::isValid($link)) {
                    $link = \App\Model\Backlink::formatLink($link);
                    if ($project->isBacklinkExists($link)) {
                        $alerts[] = 'Backlink "' . $link . '" already exists';
                        continue;
                    }
                    if ($avoidMulti && $project->isDomainExists($link)) {
                        $alerts[] = 'Link with domain '.\App\Model\Backlink::getHostWithoutWWW($link).' already exists for '.$link;
                        continue;
                    }
                    $backlink = \R::dispense('backlinks');
                    $backlink->link = $link;
                    $backlink->projects_id = 1;
                    $project->newBacklink($link);
                    \R::store($backlink);
                    $alerts[] = 'Backlink "' . $link . '" has been successfully added.';
                } else {
                    $alerts[] = 'Backlink "' . $link . '" is not valid';
                }
            }
        } else {
            $alerts[] = \t('CHANGE_LINKS_PARSER_NOT_FINISHED');
        }
        $this->app->flash('link-info', serialize($alerts));
        $this->redirect(PROJECT_FOLDER.'project/backlinks/1#alerts');
    }

    public function removeAction()
    {
        /** @var \App\Model\Project $project */
        $project = $this->getProject(1);
        if (!$project->id) {
            $this->redirect(PROJECT_FOLDER);
        }
        $lastParser = $project->getLastParser();
        if (($lastParser && $lastParser->getStatus() == \App\Model\Parser::STATUS_FINISH) || !$lastParser) {
            if ($this->request()->isPost()) {
                $id = $this->request()->post('id');
                if (is_numeric($id) && isset($project->ownBacklinks[$id])) {
                    if ($lastParser && $lastParser->id === $project->ownBacklinks[$id]->parser_id) {
                        $lastParser->can_update = false;
                        \R::store($lastParser);
                    }
                    unset($project->ownBacklinks[$id]);
                    \R::store($project);
                    $this->app->flash('link-delete', \t('BACKLINK_DELETED'));
                } else {
                    $this->app->flash('link-delete', \t('BACKLINK_REMOVE_INVALID_ID'));
                }
            }
        } else {
            $this->app->flash('link-delete', \t('CHANGE_LINKS_PARSER_NOT_FINISHED'));
        }
        $this->redirect(PROJECT_FOLDER.'project/backlinks/1');
    }

    public function exportAction()
    {
        $project = $this->getProject(1);
        if (!$project->id) {
            $this->redirect(PROJECT_FOLDER);
        }

        $header = array(
                'Link',
                'Created',
                'Updated',
                'Response code',
                'IP',
                'Twitter shares',
                'Facebook shares',
                'Google+ shares',
                'Delicious shares',
                'PR',
                'Google Cache',
                'Alexa',
                'OSE PA',
                'OSE DA',
                'Internal links',
                'External links',
                'Project link',
                'Anchor',
                'Dofollow',

        );

        $lastParser = $project->getLastParser();
        if (($lastParser && $lastParser->getStatus() === \App\Model\Parser::STATUS_FINISH) || !$lastParser )  {
            header('Content-Type: application/csv');
            header('Content-Disposition: attachement; filename="'.$project->name.'.csv";');

            $f = fopen('php://output', 'w');

            fputcsv($f, $header, ',');

            /** @var \App\Model\Backlink $backlink */
            foreach ($project->ownBacklinks as $backlink) {
                $projectLink = $backlink->getProjectLink();

                $facebookShareCount = $backlink->facebook;
                if ($facebook = @unserialize($backlink->facebook)) {
                    if (is_array($facebook)) {
                        $facebookShareCount = $facebook['share_count'];
                    }
                    else {
                        $facebookShareCount = $facebook;
                    }
                }

                $line = array(
                    $backlink->link,
                    $backlink->created,
                    $backlink->updated,
                    $backlink->code,
                    $backlink->ip,
                    $backlink->twitter,
                    $facebookShareCount,
                    $backlink->googleplus,
                    $backlink->delicious,
                    $backlink->pagerank,
                    $backlink->google_cache,
                    $backlink->alexa,
                    $backlink->osepa,
                    $backlink->oseda,
                    $backlink->internal_links,
                    $backlink->external_links,
                    $projectLink['href'],
                    $projectLink['anchor'],
                    ($projectLink['dofollow'])?'Yes':'No',

                );
                fputcsv($f, $line, ',');
            }
            exit;
        }
        else {
            $this->app->flash('export', \t('EXPORT_PARSER_NOT_FINISHED'));
            $this->redirect(PROJECT_FOLDER.'project/backlinks/1');
        }

    }


}